@@ -1,4 +1,5 @@
 import os
+import time
 from dataclasses import dataclass
 from typing import Optional, Set
 
@@ -27,6 +28,19 @@ class MeshParameters(JsonSchemaMixin):
     transform_id: str = "world"
 
 
+def wait_for(timeout: float = 10.0) -> None:
+
+    start_time = time.monotonic()
+    while time.monotonic() < start_time + timeout:
+        try:
+            started()
+            return
+        except SceneServiceException:
+            pass
+
+    raise SceneServiceException("Failed to contact Scene service.")
+
+
 @handle(SceneServiceException, message="Failed to add or update the collision model.")
 def upsert_collision(model: Models, pose: Pose, mesh_parameters: Optional[MeshParameters] = None) -> None:
     """Adds arbitrary collision model to the collision scene.
@@ -90,10 +104,11 @@ def stop() -> None:
     rest.call(rest.Method.PUT, f"{URL}/system/stop")
 
 
+@handle(SceneServiceException, message="Failed to get scene state.")
 def started() -> bool:
     """Checks whether the scene is running."""
 
-    return rest.call(rest.Method.PUT, f"{URL}/system/running", return_type=bool)
+    return rest.call(rest.Method.GET, f"{URL}/system/running", return_type=bool)
 
 
 @handle(SceneServiceException, message="Failed to get transforms.")
